/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include <dawn/dawn.h>

#define BREAKPOINT_COUNT 4

#define GRID_CHILD_COUNT 32

typedef struct {
  float width;
} breakpoint_t;

typedef struct {
  uint8_t rows;
  uint8_t columns;
  float gutterX;
  float gutterY;
} griddef_t;

typedef void gridchildresizecallback_t(
  void *user, int32_t i,
  float screenWidth, float screenHeight,
  float x, float y,
  float width, float height
);

typedef struct {
  uint8_t columns;
  uint8_t rows;
  uint8_t x;
  uint8_t y;
} gridchilddef_t;

typedef struct {
  gridchilddef_t defs[BREAKPOINT_COUNT];
  uint8_t defCount;
  gridchildresizecallback_t *onResize;
} gridchild_t;

typedef struct {
  griddef_t grids[BREAKPOINT_COUNT];
  breakpoint_t breakpoints[BREAKPOINT_COUNT];
  gridchild_t children[GRID_CHILD_COUNT];
  uint8_t childCount;

  float width;
  float height;
  float x;
  float y;
  
  uint8_t breakpointCount;
  void *user;
} grid_t;


void gridInit(grid_t *grid);

uint8_t gridAddBreakpoint(
  grid_t *grid, float width,
  uint8_t rows, uint8_t columns,
  float gutterX, float gutterY
);

gridchild_t * gridAddChild(grid_t *grid);


void gridSetSize(grid_t *grid,
  float screenWidth, float screenHeight,
  float width, float height,
  float x, float y
);